require "player/player"
require "player/SpriteAnimation"
require "camera"

function love.load()
  debug = true
  
  -- Play and loop music
  boing = love.audio.newSource("music/boing.ogg", "stream")
  mario64 = love.audio.newSource("music/mario64_helloworld.ogg", "stream")
  jumpSound = love.audio.newSource("music/jump.ogg", "static")
  boing:setLooping(true)
  jumpSound:setLooping(false)
  love.audio.play(mario64)
  
  width = love.graphics.getWidth()
  height = love.graphics.getHeight()
  love.graphics.setBackgroundColor(85, 170, 255)
  playerColor = {255,0,128} -- R, G, B arrays of stuff's colour
      
  -- Build and initialise the player
  p = player:new()
    
  p.width = 20
  p.height = 32
  p.jumpSpeed = -800
  p.bigJumpSpeed = -1200
  p.runSpeed = 500
  gravity = 2000
  hasJumped = false
  delay = 120

  -- Load the map
  love.loadNewMap("map00.tmx")
  
  -- Load player animation
    animation = SpriteAnimation:new("player/sprites2.png", 20, 32, 4, 4)
    animation:load(delay)
end
 
function love.update(dt)
 -- check controls
  if love.keyboard.isDown("right") then
    p.decelarating = false
    p:moveRight()
  end
  if love.keyboard.isDown("left") then
    p.decelarating = false
    p:moveLeft()
  end
  if love.keyboard.isDown("x") and not(hasJumped) then
    hasJumped = true
    p:jump()
    p.decelarating = false
  end
 
  -- update the player's position and check for collisions
  p:update(dt, gravity, map)
    
  -- update the sprite animation
  if (p.state == "stand") then
    animation:switch(1, 4, 200)
  end
  if (p.state == "moveRight") then
    animation:switch(2, 4, 120)
  end
  if (p.state == "moveLeft") then
    animation:switch(3, 2, 120)
  end
  if (p.state == "fall") then
    animation:switch(4, 2, 120)
  end
  if (p.state == "jump") then
    animation:switch(4, 3, 120)
  end
    
  animation:update(dt)
    
  -- center the camera on the player
  camera:setPosition(math.floor(p.x - width / 2), math.floor(p.y - height / 2))
end 

function love.draw()
  camera:set()
  
  -- round down our x, y values
  local x = math.floor(p.x)
  local y = math.floor(p.y)
  
  -- draw the map
  map:draw()
 
  -- draw the player shape
  animation:draw(x - p.width / 2, y - p.height / 2)
  love.graphics.rectangle("line", x - p.width / 2, y - p.height / 2, 20, 32) 
  
  camera:unset()
  love.graphics.setColor(255, 255, 255)
  
  
  love.graphics.print("Max speed: "..p.xSpeedMax.."",400,700)
  love.graphics.print("Speed: "..p.xSpeed.."",600,700)


  -- debug information
  if debug then
    local tileX = math.floor(p.x / map.tileWidth)
    local tileY = math.floor(p.y / map.tileHeight)
    local isTrue = ""
    
    -- Print out debug stuff
    love.graphics.print("player coordinates: ("..x..","..y..")", 5, 5)
    love.graphics.print("Current state: "..p.state, 5, 20)
    love.graphics.print("Current tile: ("..tileX..", "..tileY..")", 5, 35)
    love.graphics.print("Current speed: ("..p.xSpeed..", "..p.ySpeed..")",5,50)
    love.graphics.print("Max speed: ("..p.xSpeedMax..", "..p.ySpeedMax..")",5,65)
    love.graphics.print("Accelaration/Gravity/Jump: "..p.accelaration.." / "..gravity.." / "..p.jumpSpeed.."", 5,80)
    love.graphics.print("Current level: ("..p.level..")",5,95)
  end
  
  -- is the player dead?
  if p.y > 1200 then
    love.graphics.print("You have fallen off the map. Press 'q' to restart.", 100, 100)
  end
end

function love.keyreleased(key)
  if key == "escape" then
    love.event.push("quit")  -- quit program
  end
  if key == "`" then
    debug = not debug
  end
  if (key == "x") then
    hasJumped = false
  end    
  if (key == "right") or (key == "left") then
    p:stop() 
  end
  if key == "q" then
    p:resetXY()
  end
  
end

function math.clamp(x, min, max)
  -- too confusing
  --return x < min and min or (x > max and max or x)
    if x < min then
        return min
    elseif x >= min and x <= max then
        return x
    elseif x > max then
        return max
    end
end

function love.loadNewMap(mapName)
  -- Load up the map
  loader = require("AdvTiledLoader.Loader")
  loader.path = "maps-terrain/"
  map = loader.load(mapName)
  map:setDrawRange(0, 0, map.width * map.tileWidth, map.height * map.tileHeight)
    
  -- restrict the camera
  camera:setBounds(0, 0, map.width * map.tileWidth - width, map.height * map.tileHeight - height) 
  
  -- Change around the music
  if (p.level == 2) then
    love.audio.stop(mario64)
    love.audio.play(boing)
    mario64:setLooping(true)
  end
  
  p.x = 64
  p.y = 64
  p.xSpeed = 0
  p.ySpeed = 0
  if (p.xSpeedMaxMax < 650) then
    p.xSpeedMaxMax = p.xSpeedMaxMax + 50
    --if (p.xSpeedMaxMax == p.xSpeedMax) then
    --  p.accelaration = p.accelaration + 1
    --end
  end  
end

